@@ -1,3 +1,4 @@
+import json
 from os import rename
 import re
 import subprocess
@@ -26,19 +27,21 @@ DEFAULT_REDIRAFFE_TEMPLATE = Template(
         </noscript>
     </head>
     <body>
-        <p>You should have been redirected.</p>
-        <a href="{{rel_url}}">If not, click here to continue.</a>
         <script>
             window.location.href = '{{rel_url}}' + (window.location.search || '') + (window.location.hash || '');
         </script>
+        <p>You should have been redirected.</p>
+        <a href="{{rel_url}}">If not, click here to continue.</a>
     </body>
 </html>
 
 """
 )
-
+REDIRECT_JSON_NAME = "_rediraffe_redirected.json"
 RE_OBJ = re.compile(r"(?:(\"|')(.*?)\1|(\S+))\s+(?:(\"|')(.*?)\4|(\S+))")
 
+READTHEDOCS_BUILDERS = ["readthedocs", "readthedocsdirhtml"]
+
 
 def create_graph(path: Path) -> Dict[str, str]:
     """
@@ -47,9 +50,9 @@ def create_graph(path: Path) -> Dict[str, str]:
     graph_edges = {}
     broken = False
     with open(path, "r") as file:
-        for line_num, line in enumerate(file):
+        for line_num, line in enumerate(file, start=1):
             line = line.strip()
-            if len(line) == 0:
+            if len(line) == 0 or line.startswith("#"):
                 continue
             match = RE_OBJ.fullmatch(line)
 
@@ -122,10 +125,11 @@ def build_redirects(app: Sphinx, exception: Union[Exception, None]) -> None:
     """
     Build amd write redirects
     """
-    try:
-        app.env.redirected
-    except AttributeError:
-        app.env.redirected = {}
+    redirect_json_file = Path(app.outdir) / REDIRECT_JSON_NAME
+    if redirect_json_file.exists():
+        redirect_record = json.loads(redirect_json_file.read_text("utf8"))
+    else:
+        redirect_record = {}
 
     if exception != None:
         return
@@ -134,9 +138,12 @@ def build_redirects(app: Sphinx, exception: Union[Exception, None]) -> None:
         logger.info("rediraffe: Redirect generation skipped for linkcheck builders.")
         return
 
-    if type(app.builder) not in (StandaloneHTMLBuilder, DirectoryHTMLBuilder):
+    if (
+        type(app.builder) not in (StandaloneHTMLBuilder, DirectoryHTMLBuilder)
+        and app.builder.name not in READTHEDOCS_BUILDERS
+    ):
         logger.info(
-            "rediraffe: Redirect generation skipped for unsupported builders. Supported builders: html, dirhtml"
+            "rediraffe: Redirect generation skipped for unsupported builders. Supported builders: html, dirhtml, readthedocs, readthedocsdirhtml."
         )
         return
 
@@ -204,10 +211,9 @@ def build_redirects(app: Sphinx, exception: Union[Exception, None]) -> None:
         redirect_to = src_redirect_to.with_suffix("")
 
         if type(app.builder) == DirectoryHTMLBuilder:
-            master_doc = Path(app.config.master_doc).with_suffix("")
-            if redirect_from != master_doc:
+            if redirect_from.name != "index":
                 redirect_from = redirect_from / "index"
-            if redirect_to != master_doc:
+            if redirect_to.name != "index":
                 redirect_to = redirect_to / "index"
 
         redirect_from = redirect_from.with_suffix(".html")
@@ -219,11 +225,11 @@ def build_redirects(app: Sphinx, exception: Union[Exception, None]) -> None:
 
         if (
             build_redirect_from.exists()
-            and src_redirect_from.as_posix() in app.env.redirected
+            and src_redirect_from.as_posix() in redirect_record
         ):
             # if it is still pointing to the same source, continue
             if (
-                app.env.redirected[src_redirect_from.as_posix()]
+                redirect_record[src_redirect_from.as_posix()]
                 == src_redirect_to.as_posix()
             ):
                 continue
@@ -235,12 +241,14 @@ def build_redirects(app: Sphinx, exception: Union[Exception, None]) -> None:
                 f'{yellow("(broken)")} {redirect_from} redirects to {redirect_to} but {build_redirect_from} already exists!'
             )
             app.statuscode = 1
+            continue
 
         if not build_redirect_to.exists():
             logger.warning(
                 f'{yellow("(broken)")} {redirect_from} redirects to {redirect_to} but {build_redirect_to} does not exist!'
             )
             app.statuscode = 1
+            continue
 
         build_redirect_from.parent.mkdir(parents=True, exist_ok=True)
         with build_redirect_from.open("w") as f:
@@ -262,9 +270,9 @@ def build_redirects(app: Sphinx, exception: Union[Exception, None]) -> None:
             logger.info(
                 f'{green("(good)")} {redirect_from} {green("-->")} {redirect_to}'
             )
-            app.env.redirected[
-                src_redirect_from.as_posix()
-            ] = src_redirect_to.as_posix()
+            redirect_record[src_redirect_from.as_posix()] = src_redirect_to.as_posix()
+
+    redirect_json_file.write_text(json.dumps(redirect_record), encoding="utf8")
 
 
 class CheckRedirectsDiffBuilder(Builder):
